/* -*-c-*-
 *
 * $Id: acl.h,v 1.2 1999/07/26 23:28:16 mdw Exp $
 *
 * Access control list handling
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: acl.h,v $
 * Revision 1.2  1999/07/26 23:28:16  mdw
 * Minor modifications for new design.
 *
 * Revision 1.1.1.1  1999/07/01 08:56:23  mdw
 * Initial revision.
 *
 */

#ifndef ACL_H
#define ACL_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>

#include <sys/socket.h>
#include <netinet/in.h>

/*----- Data structures ---------------------------------------------------*/

/* --- An access control entry --- */

typedef struct acl_entry {
  struct acl_entry *next;		/* Next entry in the list */
  unsigned act;				/* What to do with matching hosts */
  struct in_addr addr, mask;		/* Address and netmask */
} acl_entry;

#define ACL_DENY 0			/* Deny access to matching hosts */
#define ACL_ALLOW 1			/* Allow access to matching hosts */
#define ACL_PERM 1u			/* Bit mask for permission bit */

/*----- Functions provided ------------------------------------------------*/

/* --- @acl_check@ --- *
 *
 * Arguments:	@acl_entry *a@ = pointer to ACL to check against
 *		@struct in_addr addr@ = address to check
 *
 * Returns:	Nonzero if allowed.
 *
 * Use:		Checks an address against an ACL.
 */

extern int acl_check(acl_entry */*a*/, struct in_addr /*addr*/);

/* --- @acl_dump@ --- *
 *
 * Arguments:	@acl_entry *a@ = pointer to ACL to dump
 *		@FILE *fp@ = pointer to stream to dump on
 *
 * Returns:	---
 *
 * Use:		Dumps an access control list to an output stream.
 */

extern void acl_dump(acl_entry */*a*/, FILE */*fp*/);

/* --- @acl_free@ --- *
 *
 * Arguments:	@acl_entry *a@ = pointer to a list of ACLs
 *
 * Returns:	---
 *
 * Use:		Frees all of the memory used by an ACL.
 */

extern void acl_free(acl_entry */*a*/);

/* --- @acl_add@ --- *
 *
 * Arguments:	@acl_entry ***a@ = address of pointer to list tail
 *		@unsigned act@ = what to do with matching addresses
 *		@struct in_addr addr, mask@ = address and mask to match
 *
 * Returns:	---
 *
 * Use:		Adds an entry to the end of an access control list.
 */

extern void acl_add(acl_entry ***/*a*/, unsigned /*act*/,
		    struct in_addr /*addr*/, struct in_addr /*mask*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
