/* -*-c-*-
 *
 * $Id: ident.c,v 1.2 1999/07/26 23:26:44 mdw Exp $
 *
 * Parse replies from RFC931 servers
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of the `fw' port forwarder.
 *
 * `fw' is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * `fw' is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with `fw'; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: ident.c,v $
 * Revision 1.2  1999/07/26 23:26:44  mdw
 * Change copyright notice.
 *
 * Revision 1.1.1.1  1999/07/01 08:56:23  mdw
 * Initial revision.
 *
 * Revision 1.1.1.1  1999/05/21 22:22:27  mdw
 * Initial import.  Code not really finished yet.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ident.h"

/*----- Main code ---------------------------------------------------------*/

/* --- @next@ --- *
 *
 * Arguments:	@char **pp@ = address of string pointer
 *
 * Returns:	Address of next token.
 *
 * Use:		Reads the next token from the result string.  Tokens are
 *		terminated by whitespace, or `:' or `,' characters.  The
 *		result string has had `\' escapes removed; it's stored in
 *		the same memory as the input string.  The actual content
 *		of the delimiter doesn't seem to be interesting, so it
 *		gets thrown away.
 */

static char *next(char **pp)
{
  char *p, *q, *r;

  /* --- Deal with reads past the end of the string --- */

  if (!*pp)
    return ("");

  /* --- Initialize various pointers into the string --- */

  p = q = r = *pp;

  /* --- Skip past any leading whitespace --- */

  while (isspace((unsigned char)*p))
    p++;

  /* --- Now start work on the string itself --- */

  for (;;) {
    if (*p == 0 || *p == ':' || *p == ',' || isspace((unsigned char)*p))
      break;
    else if (*p == '\\') {
      p++;
      if (!*p) {
	*q++ = '\\';
	break;
      }
    }
    *q++ = *p++;
  }

  /* --- Tidy up afterwards --- */

  while (isspace((unsigned char)*p))
    p++;
  if (*p == 0)
    *pp = 0;
  else if (*p == ':' || *p == ',')
    *pp = p + 1;
  else
    *pp = p;
  *q = 0;

  return (r);
}

/* --- @ident_parse@ --- *
 *
 * Arguments:	@char *p@ = pointer to input string from identd
 *		@ident *i@ = pointer to output block
 *
 * Returns:	---
 *
 * Use:		Parses a result string from an RFC931 (identd) server.
 */

void ident_parse(char *p, ident *i)
{
  char *q;

  /* --- Read the source and destination port numbers --- */

  i->sport = atoi(next(&p));
  i->dport = atoi(next(&p));

  /* --- Find out what sort of a reply this is --- */

  q = next(&p);
  if (strcmp(q, "USERID") == 0) {
    i->type = ident_userid;
    i->u.userid.os = next(&p);
    i->u.userid.user = next(&p);
  } else if (strcmp(q, "ERROR") == 0) {
    i->type = ident_error;
    i->u.error = next(&p);
  } else
    i->type = ident_bad;
}

/*----- That's all, folks -------------------------------------------------*/
